@@ -1,9 +1,12 @@
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
-# For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
-# Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
+# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
+# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt
+
+from __future__ import annotations
 
 try:
     import isort.api
+    import isort.settings
 
     HAS_ISORT_5 = True
 except ImportError:  # isort < 5
@@ -11,17 +14,20 @@ except ImportError:  # isort < 5
 
     HAS_ISORT_5 = False
 
+import argparse
 import codecs
 import os
 import re
 import sys
 import textwrap
 import tokenize
+import warnings
+from collections.abc import Sequence
 from io import BufferedReader, BytesIO
 from typing import (
     TYPE_CHECKING,
+    Any,
     List,
-    Optional,
     Pattern,
     TextIO,
     Tuple,
@@ -33,6 +39,7 @@ from typing import (
 from astroid import Module, modutils, nodes
 
 from pylint.constants import PY_EXTS
+from pylint.typing import OptionDict
 
 if sys.version_info >= (3, 8):
     from typing import Literal
@@ -41,12 +48,12 @@ else:
 
 if TYPE_CHECKING:
     from pylint.checkers.base_checker import BaseChecker
+    from pylint.lint import PyLinter
 
 DEFAULT_LINE_LENGTH = 79
 
 # These are types used to overload get_global_option() and refer to the options type
 GLOBAL_OPTION_BOOL = Literal[
-    "ignore-mixin-members",
     "suggestion-mode",
     "analyse-fallback-blocks",
     "allow-global-unused-variables",
@@ -80,7 +87,9 @@ T_GlobalOptionReturnTypes = TypeVar(
 )
 
 
-def normalize_text(text, line_len=DEFAULT_LINE_LENGTH, indent=""):
+def normalize_text(
+    text: str, line_len: int = DEFAULT_LINE_LENGTH, indent: str = ""
+) -> str:
     """Wrap the text on the given line length."""
     return "\n".join(
         textwrap.wrap(
@@ -93,20 +102,20 @@ CMPS = ["=", "-", "+"]
 
 
 # py3k has no more cmp builtin
-def cmp(a, b):
+def cmp(a: int | float, b: int | float) -> int:
     return (a > b) - (a < b)
 
 
-def diff_string(old, new):
+def diff_string(old: int | float, new: int | float) -> str:
     """Given an old and new int value, return a string representing the
-    difference
+    difference.
     """
     diff = abs(old - new)
     diff_str = f"{CMPS[cmp(old, new)]}{diff and f'{diff:.2f}' or ''}"
     return diff_str
 
 
-def get_module_and_frameid(node):
+def get_module_and_frameid(node: nodes.NodeNG) -> tuple[str, str]:
     """Return the module name and the frame id in the module."""
     frame = node.frame(future=True)
     module, obj = "", []
@@ -123,12 +132,18 @@ def get_module_and_frameid(node):
     return module, ".".join(obj)
 
 
-def get_rst_title(title, character):
-    """Permit to get a title formatted as ReStructuredText test (underlined with a chosen character)."""
+def get_rst_title(title: str, character: str) -> str:
+    """Permit to get a title formatted as ReStructuredText test (underlined with a
+    chosen character).
+    """
     return f"{title}\n{character * len(title)}\n"
 
 
-def get_rst_section(section, options, doc=None):
+def get_rst_section(
+    section: str | None,
+    options: list[tuple[str, OptionDict, Any]],
+    doc: str | None = None,
+) -> str:
     """Format an option's section using as a ReStructuredText formatted output."""
     result = ""
     if section:
@@ -140,6 +155,7 @@ def get_rst_section(section, options, doc=None):
         help_opt = optdict.get("help")
         result += f":{optname}:\n"
         if help_opt:
+            assert isinstance(help_opt, str)
             formatted_help = normalize_text(help_opt, indent="  ")
             result += f"{formatted_help}\n"
         if value and optname != "py-version":
@@ -149,7 +165,7 @@ def get_rst_section(section, options, doc=None):
 
 
 def decoding_stream(
-    stream: Union[BufferedReader, BytesIO],
+    stream: BufferedReader | BytesIO,
     encoding: str,
     errors: Literal["strict"] = "strict",
 ) -> codecs.StreamReader:
@@ -160,15 +176,15 @@ def decoding_stream(
     return reader_cls(stream, errors)
 
 
-def tokenize_module(node: nodes.Module) -> List[tokenize.TokenInfo]:
+def tokenize_module(node: nodes.Module) -> list[tokenize.TokenInfo]:
     with node.stream() as stream:
         readline = stream.readline
         return list(tokenize.tokenize(readline))
 
 
-def register_plugins(linter, directory):
+def register_plugins(linter: PyLinter, directory: str) -> None:
     """Load all module and package in the given directory, looking for a
-    'register' function in each one, used to register pylint checkers
+    'register' function in each one, used to register pylint checkers.
     """
     imported = {}
     for filename in os.listdir(directory):
@@ -189,7 +205,7 @@ def register_plugins(linter, directory):
                     os.path.join(directory, filename)
                 )
             except ValueError:
-                # empty module name (usually emacs auto-save files)
+                # empty module name (usually Emacs auto-save files)
                 continue
             except ImportError as exc:
                 print(f"Problem importing module {filename}: {exc}", file=sys.stderr)
@@ -201,81 +217,76 @@ def register_plugins(linter, directory):
 
 @overload
 def get_global_option(
-    checker: "BaseChecker", option: GLOBAL_OPTION_BOOL, default: Optional[bool] = None
+    checker: BaseChecker, option: GLOBAL_OPTION_BOOL, default: bool | None = ...
 ) -> bool:
     ...
 
 
 @overload
 def get_global_option(
-    checker: "BaseChecker", option: GLOBAL_OPTION_INT, default: Optional[int] = None
+    checker: BaseChecker, option: GLOBAL_OPTION_INT, default: int | None = ...
 ) -> int:
     ...
 
 
 @overload
 def get_global_option(
-    checker: "BaseChecker",
+    checker: BaseChecker,
     option: GLOBAL_OPTION_LIST,
-    default: Optional[List[str]] = None,
-) -> List[str]:
+    default: list[str] | None = ...,
+) -> list[str]:
     ...
 
 
 @overload
 def get_global_option(
-    checker: "BaseChecker",
+    checker: BaseChecker,
     option: GLOBAL_OPTION_PATTERN,
-    default: Optional[Pattern[str]] = None,
+    default: Pattern[str] | None = ...,
 ) -> Pattern[str]:
     ...
 
 
 @overload
 def get_global_option(
-    checker: "BaseChecker",
+    checker: BaseChecker,
     option: GLOBAL_OPTION_PATTERN_LIST,
-    default: Optional[List[Pattern[str]]] = None,
-) -> List[Pattern[str]]:
+    default: list[Pattern[str]] | None = ...,
+) -> list[Pattern[str]]:
     ...
 
 
 @overload
 def get_global_option(
-    checker: "BaseChecker",
+    checker: BaseChecker,
     option: GLOBAL_OPTION_TUPLE_INT,
-    default: Optional[Tuple[int, ...]] = None,
-) -> Tuple[int, ...]:
+    default: tuple[int, ...] | None = ...,
+) -> tuple[int, ...]:
     ...
 
 
 def get_global_option(
-    checker: "BaseChecker",
+    checker: BaseChecker,
     option: GLOBAL_OPTION_NAMES,
-    default: Optional[T_GlobalOptionReturnTypes] = None,
-) -> Optional[T_GlobalOptionReturnTypes]:
-    """Retrieve an option defined by the given *checker* or
+    default: T_GlobalOptionReturnTypes | None = None,  # pylint: disable=unused-argument
+) -> T_GlobalOptionReturnTypes | None | Any:
+    """DEPRECATED: Retrieve an option defined by the given *checker* or
     by all known option providers.
 
     It will look in the list of all options providers
     until the given *option* will be found.
     If the option wasn't found, the *default* value will be returned.
     """
-    # First, try in the given checker's config.
-    # After that, look in the options providers.
-
-    try:
-        return getattr(checker.config, option.replace("-", "_"))
-    except AttributeError:
-        pass
-    for provider in checker.linter.options_providers:
-        for options in provider.options:
-            if options[0] == option:
-                return getattr(provider.config, option.replace("-", "_"))
-    return default
+    warnings.warn(
+        "get_global_option has been deprecated. You can use "
+        "checker.linter.config to get all global options instead.",
+        DeprecationWarning,
+        stacklevel=2,
+    )
+    return getattr(checker.linter.config, option.replace("-", "_"))
 
 
-def _splitstrip(string, sep=","):
+def _splitstrip(string: str, sep: str = ",") -> list[str]:
     """Return a list of stripped string by splitting the string given as
     argument on `sep` (',' by default), empty strings are discarded.
 
@@ -298,13 +309,10 @@ def _splitstrip(string, sep=","):
     return [word.strip() for word in string.split(sep) if word.strip()]
 
 
-def _unquote(string):
+def _unquote(string: str) -> str:
     """Remove optional quotes (simple or double) from the string.
 
-    :type string: str or unicode
     :param string: an optionally quoted string
-
-    :rtype: str or unicode
     :return: the unquoted string (or the input string if it wasn't quoted)
     """
     if not string:
@@ -316,7 +324,7 @@ def _unquote(string):
     return string
 
 
-def _check_csv(value):
+def _check_csv(value: list[str] | tuple[str] | str) -> Sequence[str]:
     if isinstance(value, (list, tuple)):
         return value
     return _splitstrip(value)
@@ -329,8 +337,11 @@ def _comment(string: str) -> str:
     return "# " + f"{sep}# ".join(lines)
 
 
-def _format_option_value(optdict, value):
-    """Return the user input's value from a 'compiled' value."""
+def _format_option_value(optdict: OptionDict, value: Any) -> str:
+    """Return the user input's value from a 'compiled' value.
+
+    TODO: 3.0: Remove deprecated function
+    """
     if optdict.get("type", None) == "py_version":
         value = ".".join(str(item) for item in value)
     elif isinstance(value, (list, tuple)):
@@ -344,31 +355,52 @@ def _format_option_value(optdict, value):
         value = "yes" if value else "no"
     elif isinstance(value, str) and value.isspace():
         value = f"'{value}'"
-    return value
+    return str(value)
 
 
 def format_section(
-    stream: TextIO, section: str, options: List[Tuple], doc: Optional[str] = None
+    stream: TextIO,
+    section: str,
+    options: list[tuple[str, OptionDict, Any]],
+    doc: str | None = None,
 ) -> None:
     """Format an option's section using the INI format."""
+    warnings.warn(
+        "format_section has been deprecated. It will be removed in pylint 3.0.",
+        DeprecationWarning,
+        stacklevel=2,
+    )
     if doc:
         print(_comment(doc), file=stream)
     print(f"[{section}]", file=stream)
-    _ini_format(stream, options)
+    with warnings.catch_warnings():
+        warnings.filterwarnings("ignore", category=DeprecationWarning)
+        _ini_format(stream, options)
 
 
-def _ini_format(stream: TextIO, options: List[Tuple]) -> None:
+def _ini_format(stream: TextIO, options: list[tuple[str, OptionDict, Any]]) -> None:
     """Format options using the INI format."""
+    warnings.warn(
+        "_ini_format has been deprecated. It will be removed in pylint 3.0.",
+        DeprecationWarning,
+        stacklevel=2,
+    )
     for optname, optdict, value in options:
+        # Skip deprecated option
+        if "kwargs" in optdict:
+            assert isinstance(optdict["kwargs"], dict)
+            if "new_names" in optdict["kwargs"]:
+                continue
         value = _format_option_value(optdict, value)
         help_opt = optdict.get("help")
         if help_opt:
+            assert isinstance(help_opt, str)
             help_opt = normalize_text(help_opt, indent="# ")
             print(file=stream)
             print(help_opt, file=stream)
         else:
             print(file=stream)
-        if value is None:
+        if value in {"None", "False"}:
             print(f"#{optname}=", file=stream)
         else:
             value = str(value).strip()
@@ -383,9 +415,9 @@ def _ini_format(stream: TextIO, options: List[Tuple]) -> None:
 class IsortDriver:
     """A wrapper around isort API that changed between versions 4 and 5."""
 
-    def __init__(self, config):
+    def __init__(self, config: argparse.Namespace) -> None:
         if HAS_ISORT_5:
-            self.isort5_config = isort.api.Config(
+            self.isort5_config = isort.settings.Config(
                 # There is no typo here. EXTRA_standard_library is
                 # what most users want. The option has been named
                 # KNOWN_standard_library for ages in pylint, and we
@@ -394,13 +426,14 @@ class IsortDriver:
                 known_third_party=config.known_third_party,
             )
         else:
-            self.isort4_obj = isort.SortImports(  # pylint: disable=no-member
+            # pylint: disable-next=no-member
+            self.isort4_obj = isort.SortImports(  # type: ignore[attr-defined]
                 file_contents="",
                 known_standard_library=config.known_standard_library,
                 known_third_party=config.known_third_party,
             )
 
-    def place_module(self, package):
+    def place_module(self, package: str) -> str:
         if HAS_ISORT_5:
             return isort.api.place_module(package, self.isort5_config)
-        return self.isort4_obj.place_module(package)
+        return self.isort4_obj.place_module(package)  # type: ignore[no-any-return]
